@@ -4,7 +4,7 @@
 from typing import Any, Dict, Optional
 
 import vapoursynth as vs
-from vsutil import depth
+from vsutil import depth, fallback
 
 from . import denoise
 
@@ -13,6 +13,7 @@ core = vs.core
 
 def bidehalo(clip: vs.VideoNode, ref: Optional[vs.VideoNode] = None,
              sigmaS: float = 1.5, sigmaR: float = 5/255,
+             sigmaS_final: Optional[float] = None, sigmaR_final: Optional[float] = None,
              bilateral_args: Dict[str, Any] = {},
              bm3d_args: Dict[str, Any] = {},
              ) -> vs.VideoNode:
@@ -23,9 +24,14 @@ def bidehalo(clip: vs.VideoNode, ref: Optional[vs.VideoNode] = None,
     :param clip:                Source clip
     :param ref:                 Ref clip
     :param sigmaS:              Bilateral's spatial weight sigma
-    :param sigmaS:              Bilateral's range weight sigma
+    :param sigmaR:              Bilateral's range weight sigma
+    :param sigmaS_final:        Final bilateral call's spatial weight sigma.
+                                You'll want this to be much weaker than the initial `sigmaS`.
+                                If `None`, 1/3rd of `sigmaS`.
+    :param sigmaR_final:        Bilateral's range weight sigma.
+                                if `None`, same as `sigmaR`
     :param bilateral_args:      Additional parameters to pass to bilateral
-    :param bm3d_args:           Additional parameters to pass to denoise.bm3d
+    :param bm3d_args:           Additional parameters to pass to :py:class:`lvsfunc.denoise.bm3d`
 
     :return:                    Dehalo'd clip
     """
@@ -33,12 +39,16 @@ def bidehalo(clip: vs.VideoNode, ref: Optional[vs.VideoNode] = None,
     bm3ddh_args.update(bm3d_args)
 
     if clip.format is None:
-        raise ValueError("lfdeband: 'Variable-format clips not supported'")
+        raise ValueError("bidehalo: 'Variable-format clips not supported'")
+
+    sigmaS_final = fallback(sigmaS_final, sigmaS / 3)
+    sigmaR_final = fallback(sigmaR_final, sigmaR)
 
     if ref is None:
-        bm3ddh = depth(denoise.bm3d(clip, **bm3ddh_args), 16)
-        bidh_ref = core.bilateral.Bilateral(bm3ddh, sigmaS=sigmaS, sigmaR=sigmaR, **bilateral_args)
-        bidh = core.bilateral.Bilateral(bm3ddh, ref=bidh_ref, sigmaS=sigmaS, sigmaR=sigmaR, **bilateral_args)
+        den = depth(denoise.bm3d(clip, **bm3ddh_args), 16)
+
+        ref = den.bilateral.Bilateral(sigmaS=sigmaS, sigmaR=sigmaR, **bilateral_args)
+        bidh = den.bilateral.Bilateral(ref=ref, sigmaS=sigmaS_final, sigmaR=sigmaR_final, **bilateral_args)
         bidh = depth(bidh, clip.format.bits_per_sample)
     else:
         bidh = depth(ref, clip.format.bits_per_sample)
