@@ -2,12 +2,16 @@
     Functions to help with deinterlacing or deinterlaced content.
 """
 
+import os
+import time
 from functools import partial
-from typing import Any, Optional
+from pathlib import Path
+from typing import Any, Optional, Union, Dict
 
 import vapoursynth as vs
 
 from . import util
+from .render import get_render_progress
 
 core = vs.core
 
@@ -35,6 +39,70 @@ def SIVTC(clip: vs.VideoNode, pattern: int = 0,
     return core.std.SetFrameProp(dec, prop='_FieldBased', intval=0)
 
 
+def TIVTC_VFR(clip: vs.VideoNode,
+              tfm_in: Union[Path, str] = ".ivtc/matches.txt",
+              tdec_in: Union[Path, str] = ".ivtc/metrics.txt",
+              timecodes_out: Union[Path, str] = ".ivtc/timecodes.txt",
+              tfm_args: Dict[str, Any] = {},
+              tdecimate_args: Dict[str, Any] = {}) -> vs.VideoNode:
+    """
+    Wrapper for performing TFM and TDecimate on a clip that is supposed to be VFR,
+    including generating a metrics/matches/timecodes txt file.
+
+    Largely based on, if not basically rewritten from, atomchtools.TIVTC_VFR.
+
+    Dependencies:
+
+    * TIVTC
+
+    :param clip:                Input clip
+    :param tfmIn:               File location for TFM's matches analysis
+    :param tdecIn:              File location for TDecimate's metrics analysis
+    :param mkvOut:              File location for TDecimate's timecode file output
+    :param tfm_args:            Additional arguments to pass to TFM
+    :param tdecimate_args:      Additional arguments to pass to TDecimate
+
+    :return:                    IVTC'd VFR clip
+    """
+    tfm_in = Path(tfm_in).resolve()
+    tdec_in = Path(tdec_in).resolve()
+    timecodes_out = Path(timecodes_out).resolve()
+
+    # TIVTC can't write files into directories that don't exist
+    for p in (tfm_in, tdec_in, timecodes_out):
+        if not p.parent.exists():
+            p.parent.mkdir(parents=True)
+
+    if not (tfm_in.exists() and tdec_in.exists()):
+        tfm_analysis: Dict[str, Any] = {**tfm_args, 'output': str(tfm_in)}
+        tdec_analysis: Dict[str, Any] = {**tdecimate_args, 'output': str(tdec_in), 'mode': 4}
+
+        ivtc_clip = core.tivtc.TFM(clip, **tfm_analysis).tivtc.TDecimate(**tdec_analysis)
+
+        with get_render_progress() as pr:
+            task = pr.add_task("Analyzing frames...", total=ivtc_clip.num_frames)
+
+            def _cb(n: int, total: int) -> None:
+                pr.update(task, advance=1)
+
+            with open(os.devnull, 'wb') as dn:
+                ivtc_clip.output(dn, progress_update=_cb)
+
+        # Allow it to properly finish writing the logs
+        time.sleep(0.5)  # TO-DO: if possible, make it forcibly refresh the core instance so it stops throwing an error
+        raise vs.Error("TIVTC_VFR: vs.core must be refreshed after the analysis pass. Please refresh your preview")
+
+    tfm_args = {**tfm_args, 'input': str(tfm_in)}
+
+    tdecimate_args = {
+        'mode': 5, **tdecimate_args,
+        'input': str(tdec_in), 'tfmIn': str(tfm_in), 'mkvOut': str(timecodes_out),
+        'hybrid': 2, 'vfrDec': 1
+    }
+
+    return clip.tivtc.TFM(**tfm_args).tivtc.TDecimate(**tdecimate_args)
+
+
 def deblend(clip: vs.VideoNode, rep: Optional[int] = None) -> vs.VideoNode:
     """
     A simple function to fix deblending for interlaced video with an AABBA blending pattern,
@@ -51,6 +119,7 @@ def deblend(clip: vs.VideoNode, rep: Optional[int] = None) -> vs.VideoNode:
     https://mechaweaponsvidya.wordpress.com/2012/09/13/adventures-in-deblending/
 
     Dependencies:
+
     * RGSF (optional: 32 bit clip)
 
     :param clip:     Input clip
@@ -99,6 +168,7 @@ def decomb(clip: vs.VideoNode,
     Base function written by Midlifecrisis from the WEEB AUTISM server, and modified by LightArrowsEXE.
 
     Dependencies:
+
     * combmask
     * havsfunc
     * RGSF (optional: 32 bit clip)
@@ -165,6 +235,7 @@ def dir_deshimmer(clip: vs.VideoNode, TFF: bool = True,
     Odds of success are low. But if you're desperate, it's worth a shot.
 
     Dependencies:
+
     * vapoursynth-nnedi3
 
     :param clip:         Input clip
@@ -193,6 +264,7 @@ def dir_unsharp(clip: vs.VideoNode,
     Special thanks to thebombzen and kageru for writing the bulk of this.
 
     Dependencies:
+
     * knlmeanscl
 
     :param clip:            Input clip
