@@ -64,6 +64,7 @@ def finish_frame(outfile: Optional[BinaryIO], timecodes: Optional[TextIO], ctx:
 def clip_async_render(clip: vs.VideoNode,
                       outfile: Optional[BinaryIO] = None,
                       timecodes: Optional[TextIO] = None,
+                      progress: Optional[str] = "Rendering clip...",
                       callback: Union[RenderCallback, List[RenderCallback], None] = None) -> List[float]:
     """
     Render a clip by requesting frames asynchronously using clip.get_frame_async,
@@ -77,6 +78,8 @@ def clip_async_render(clip: vs.VideoNode,
     :param outfile:   Y4MPEG render output BinaryIO handle. If None, no Y4M output is performed.
                       Use ``sys.stdout.buffer`` for stdout. (Default: None)
     :param timecodes: Timecode v2 file TextIO handle. If None, timecodes will not be written.
+    :param progress:  String to use for render progress display.
+                      If empty or ``None``, no progress display.
     :param callback:  Single or list of callbacks to be preformed. The callbacks are called
                       when each sequential frame is output, not when each frame is done.
                       Must have signature ``Callable[[int, vs.VideoNode], None]``
@@ -86,17 +89,41 @@ def clip_async_render(clip: vs.VideoNode,
     """
     cbl = [] if callback is None else callback if isinstance(callback, list) else [callback]
 
+    if progress:
+        p = get_render_progress()
+        task = p.add_task(progress, total=clip.num_frames)
+
+        def _progress_cb(n: int, f: vs.VideoFrame) -> None:
+            p.update(task, advance=1)
+
+        cbl.append(_progress_cb)
+
     ctx = RenderContext(clip, core.num_threads)
 
+    bad_timecodes: bool = False
+
     def cb(f: Future[vs.VideoFrame], n: int) -> None:
         ctx.frames[n] = f.result()
         nn = ctx.queued
 
         while ctx.frames_rendered in ctx.frames:
+            nonlocal timecodes
+            nonlocal bad_timecodes
+
             frame = ctx.frames[ctx.frames_rendered]
-            ctx.timecodes.append(ctx.timecodes[-1]
-                                 + get_prop(frame, "_DurationNum", int)
-                                 / get_prop(frame, "_DurationDen", int))
+            # if a frame is missing timing info, clear timecodes because they're worthless
+            if ("_DurationNum" not in frame.props or "_DurationDen" not in frame.props) and not bad_timecodes:
+                bad_timecodes = True
+                if timecodes:
+                    timecodes.seek(0)
+                    timecodes.truncate()
+                    timecodes = None
+                ctx.timecodes = []
+                print("clip_async_render: frame missing duration information, discarding timecodes")
+            elif not bad_timecodes:
+                ctx.timecodes.append(ctx.timecodes[-1]
+                                     + get_prop(frame, "_DurationNum", int)
+                                     / get_prop(frame, "_DurationDen", int))
             finish_frame(outfile, timecodes, ctx)
             [cb(ctx.frames_rendered, ctx.frames[ctx.frames_rendered]) for cb in cbl]
             del ctx.frames[ctx.frames_rendered]  # tfw no infinite memory
@@ -147,16 +174,33 @@ def clip_async_render(clip: vs.VideoNode,
     ctx.condition.acquire()
 
     # seed threads
-    for n in range(min(clip.num_frames, core.num_threads)):
-        cbp = partial(cb, n=n)  # lambda won't bind the int immediately
-        clip.get_frame_async(n).add_done_callback(cbp)  # type: ignore
+    if progress:
+        p.start()
+    try:
+        for n in range(min(clip.num_frames, core.num_threads)):
+            cbp = partial(cb, n=n)  # lambda won't bind the int immediately
+            clip.get_frame_async(n).add_done_callback(cbp)  # type: ignore
 
-    while ctx.frames_rendered != clip.num_frames:
-        ctx.condition.wait()
+        while ctx.frames_rendered != clip.num_frames:
+            ctx.condition.wait()
+    finally:
+        if progress:
+            p.stop()
 
     return ctx.timecodes  # might as well
 
 
+def get_render_progress() -> Progress:
+    return Progress(
+        TextColumn("{task.description}"),
+        BarColumn(),
+        TextColumn("{task.completed}/{task.total}"),
+        TextColumn("{task.percentage:>3.02f}%"),
+        FPSColumn(),
+        TimeRemainingColumn(),
+    )
+
+
 class SceneChangeMode(Enum):
     WWXD = 0
     SCXVID = 1
@@ -169,6 +213,7 @@ def find_scene_changes(clip: vs.VideoNode, mode: SceneChangeMode = SceneChangeMo
     Generate a list of scene changes (keyframes).
 
     Dependencies:
+
     * vapoursynth-wwxd
     * vapoursynth-scxvid (Optional: scxvid mode)
 
